pub struct DriveInfo {
pub path: String,
pub display_name: Option<String>,
pub has_audio_cd: bool,
}Expand description
Information about all found drives. This info is not tested extensively, and in general it is encouraged to provide a disk drive directly.
Fields
path: StringPath to the drive, which can be something like ‘disk6’ on macOS, ‘\.\E:’ on Windows, and ‘/dev/sr0’ on Linux
display_name: Option<String>Just the device name, without the full path for the OS
has_audio_cd: boolWe load the disc and issue a TOC command, which is supported only on media CDs