pub struct Track {
pub number: u8,
pub start_lba: u32,
pub start_msf: (u8, u8, u8),
pub is_audio: bool,
}Expand description
Representation of the track from ToC, purely in terms of data location on the CD.
Fields
number: u8Track number from the Table of Contents (read from the CD itself). It usually starts with 1, but you should read this value directly when reading raw track data. There might be gaps, and also in the future there might be hidden track support, which will be located at number 0.
start_lba: u32starting offset
start_msf: (u8, u8, u8)Track start address in (minutes, seconds, frames) (MSF) form.
MSF uses 75 frames per second and includes the standard 150-frame
lead-in offset, so LBA 0 corresponds to (0, 2, 0). See lba_to_msf.
is_audio: boolWhether the TOC identifies this as an audio track.
A value of false indicates a data track.