pub struct DriveInfo {
pub path: String,
pub has_audio_cd: bool,
}Expand description
Information about an optical drive discovered by CdReader::list_drives.
Audio-CD detection is best-effort. If a drive cannot be opened or its TOC
cannot be read, it is still returned with DriveInfo::has_audio_cd set to
false. If you already know the platform-specific device path, you can
bypass discovery with CdReader::open_path.
Fields
path: StringPath to the drive, which can be something like ‘disk6’ on macOS, ‘\.\E:’ on Windows, and ‘/dev/sr0’ on Linux
has_audio_cd: boolWhether the current disc appears to contain at least one audio track.