Toc

Struct Toc 

pub struct Toc {
    pub first_track: u8,
    pub last_track: u8,
    pub tracks: Vec<Track>,
    pub leadout_lba: u32,
}
Table of Contents, read directly from the Audio CD. The most important part is the tracks vector, which allows you to read raw track data.

Fields

first_track: u8

Helper value with the first track number

last_track: u8

Helper value with the last track number. You should not use it directly to iterate over all available tracks, as there might be gaps.

tracks: Vec<Track>

List of tracks with LBA and MSF offsets

leadout_lba: u32

Lead-out LBA reported by the drive for the disc TOC. You’ll also need this in order to calculate MusicBrainz ID.